==> 0.3.0.tar.gz: android/src/main/kotlin/com/contentsquare/flutter/ContentsquarePlugin.kt <==
package com.contentsquare.flutter

import android.app.Activity
import android.content.Context
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.ViewGroup
import androidx.annotation.NonNull
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphListener
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphResult
import io.flutter.Log
import io.flutter.embedding.android.FlutterSurfaceView
import io.flutter.embedding.engine.plugins.lifecycle.HiddenLifecycleReference


/** ContentsquarePlugin */
class ContentsquarePlugin : FlutterPlugin, ActivityAware, MethodCallHandler,
    DefaultLifecycleObserver {
    private lateinit var channel: MethodChannel
    private var context: Context? = null
    private var methodCallHandlerImpl = MethodCallHandlerImpl()
    private lateinit var view: View
    private lateinit var activity: Activity

    private fun registerNewExternalViewFromActivity(@NonNull activity: Activity) {
        view = activity.findViewById(android.R.id.content) as View

        val flutterSurfaceView =
            (((((view as ViewGroup).getChildAt(0) as ViewGroup).getChildAt(0) as ViewGroup)).getChildAt(
                0
            ) as FlutterSurfaceView)

        MethodCallHandlerImpl.CSFlutter.registerExternalView(
            flutterSurfaceView,
            object : ExternalViewGraphListener {
                override fun takeSnapShot(
                    view: View,
                    path: String,
                    externalViewGraphResult: ExternalViewGraphResult
                ) {
                    val targetViewPath: String =
                        path + MethodCallHandlerImpl.CSFlutter.FLUTTER_SEPARATOR

                    val pixelRatio: Double =
                        context?.resources?.displayMetrics?.density?.toDouble() ?: 1.0

                    channel.invokeMethod(
                        "requestFlutterSnapshot",
                        mapOf("rootPath" to targetViewPath, "pixelRatio" to pixelRatio),
                        object : Result {
                            override fun success(result: Any?) {
                                externalViewGraphResult.onSnapshotTaken(
                                    view,
                                    result as String
                                )
                            }

                            override fun error(
                                errorCode: String?,
                                errorMessage: String?,
                                errorDetails: Any?
                            ) {
                                Log.d("Android", "$errorCode, $errorMessage, $errorDetails")
                            }

                            override fun notImplemented() {
                                Log.d("Android", "notImplemented")
                            }
                        })
                }
            })
    }

    override fun onAttachedToEngine(
        @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    ) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "contentsquare")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    override fun onAttachedToActivity(@NonNull activityPluginBinding: ActivityPluginBinding) {
        (activityPluginBinding.lifecycle as HiddenLifecycleReference).lifecycle.addObserver(this)
        activity = activityPluginBinding.activity;
    }

    override fun onResume(owner: LifecycleOwner) {
        registerNewExternalViewFromActivity(activity)
    }

    override fun onReattachedToActivityForConfigChanges(@NonNull activityPluginBinding: ActivityPluginBinding) {
        registerNewExternalViewFromActivity(activityPluginBinding.activity)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "send" -> {
                methodCallHandlerImpl.send(call, result)
            }
            "sendTransaction" -> {
                methodCallHandlerImpl.sendTransaction(call, result)
            }
            "sendDynamicVar" -> {
                methodCallHandlerImpl.sendDynamicVar(call, result)
            }
            "optIn" -> {
                methodCallHandlerImpl.optIn(result, context)
            }
            "optOut" -> {
                methodCallHandlerImpl.optOut(result, context)
            }
            "forgetMe" -> {
                methodCallHandlerImpl.forgetMe(result)
            }
            "getUserId" -> {
                methodCallHandlerImpl.getUserId(result)
            }
            "stopTracking" -> {
                methodCallHandlerImpl.stopTracking(result)
            }
            "resumeTracking" -> {
                methodCallHandlerImpl.resumeTracking(result)
            }
            "sendTouchEvent" -> {
                methodCallHandlerImpl.sendTouchEvent(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        context = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
        MethodCallHandlerImpl.CSFlutter.unRegisterExternalView(view)
    }

    override fun onDetachedFromActivity() {
        MethodCallHandlerImpl.CSFlutter.unRegisterExternalView(view)
    }
}

==> 0.4.1.tar.gz: android/src/main/kotlin/com/contentsquare/flutter/ContentsquarePlugin.kt <==
package com.contentsquare.flutter

import android.app.Activity
import android.content.Context
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.ViewGroup
import androidx.annotation.NonNull
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleOwner
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphListener
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphResult
import io.flutter.Log
import io.flutter.embedding.android.FlutterSurfaceView
import io.flutter.embedding.engine.plugins.lifecycle.HiddenLifecycleReference

internal class FlutterViewFinder {

    fun find(activity: Activity): View? {
        val view = activity.window.decorView
        return findRecursively(view)
    }

    private fun findRecursively(view: View): View? {
        if (view is FlutterSurfaceView) {
            return view
        }
        if (view is ViewGroup) {
            for (childIndex in 0 until view.childCount) {
                val viewFound = findRecursively(view.getChildAt(childIndex))
                if (viewFound != null) {
                    return viewFound
                }
            }
        }
        return null
    }
}

/** ContentsquarePlugin */
class ContentsquarePlugin : FlutterPlugin, ActivityAware, MethodCallHandler,
    DefaultLifecycleObserver {
    private lateinit var channel: MethodChannel
    private var context: Context? = null
    private var methodCallHandlerImpl = MethodCallHandlerImpl()
    private lateinit var view: View
    private lateinit var activity: Activity

    private fun registerNewExternalViewFromActivity(@NonNull activity: Activity) {
        view = activity.findViewById(android.R.id.content) as View


        val flutterSurfaceView = FlutterViewFinder().find(activity)
        if (flutterSurfaceView == null) {
            Log.d("Android", "ContentsquarePlugin, no Flutter view found.")
            return
        }
        Log.d("Android", "ContentsquarePlugin, Flutter view found $flutterSurfaceView")

        MethodCallHandlerImpl.CSFlutter.registerExternalView(
            flutterSurfaceView,
            object : ExternalViewGraphListener {
                override fun takeSnapShot(
                    view: View,
                    path: String,
                    externalViewGraphResult: ExternalViewGraphResult
                ) {
                    val targetViewPath: String =
                        path + MethodCallHandlerImpl.CSFlutter.FLUTTER_SEPARATOR

                    val pixelRatio: Double =
                        context?.resources?.displayMetrics?.density?.toDouble() ?: 1.0

                    channel.invokeMethod(
                        "requestFlutterSnapshot",
                        mapOf("rootPath" to targetViewPath, "pixelRatio" to pixelRatio),
                        object : Result {
                            override fun success(result: Any?) {
                                externalViewGraphResult.onSnapshotTaken(
                                    view,
                                    result as String
                                )
                            }

                            override fun error(
                                errorCode: String?,
                                errorMessage: String?,
                                errorDetails: Any?
                            ) {
                                Log.d("Android", "$errorCode, $errorMessage, $errorDetails")
                            }

                            override fun notImplemented() {
                                Log.d("Android", "notImplemented")
                            }
                        })
                }
            })
    }

    override fun onAttachedToEngine(
        @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    ) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "contentsquare")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    override fun onAttachedToActivity(@NonNull activityPluginBinding: ActivityPluginBinding) {
        (activityPluginBinding.lifecycle as HiddenLifecycleReference).lifecycle.addObserver(this)
        activity = activityPluginBinding.activity;
    }

    override fun onResume(owner: LifecycleOwner) {
        registerNewExternalViewFromActivity(activity)
    }

    override fun onReattachedToActivityForConfigChanges(@NonNull activityPluginBinding: ActivityPluginBinding) {
        registerNewExternalViewFromActivity(activityPluginBinding.activity)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "start" -> {
                methodCallHandlerImpl.start(result, context)
            }
            "send" -> {
                methodCallHandlerImpl.send(call, result)
            }
            "sendTransaction" -> {
                methodCallHandlerImpl.sendTransaction(call, result)
            }
            "sendDynamicVar" -> {
                methodCallHandlerImpl.sendDynamicVar(call, result)
            }
            "optIn" -> {
                methodCallHandlerImpl.optIn(result, context)
            }
            "optOut" -> {
                methodCallHandlerImpl.optOut(result, context)
            }
            "forgetMe" -> {
                methodCallHandlerImpl.forgetMe(result)
            }
            "getUserId" -> {
                methodCallHandlerImpl.getUserId(result)
            }
            "stopTracking" -> {
                methodCallHandlerImpl.stopTracking(result)
            }
            "resumeTracking" -> {
                methodCallHandlerImpl.resumeTracking(result)
            }
            "sendTouchEvent" -> {
                methodCallHandlerImpl.sendTouchEvent(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        context = null
    }

    override fun onDetachedFromActivityForConfigChanges() {
        MethodCallHandlerImpl.CSFlutter.unRegisterExternalView(view)
    }

    override fun onDetachedFromActivity() {
        MethodCallHandlerImpl.CSFlutter.unRegisterExternalView(view)
    }
}
